package main

import (
	"bufio"
	"fmt"
	"os"
	"sort"
	"strings"
)

const FILE = "/tmp/README"

func main() {
	// Open the input file
	file, err := os.Open(FILE)
	if err != nil {
		fmt.Println("Error opening file:", err)
		return
	}
	defer file.Close()

	// Create a scanner to read the file line by line
	scanner := bufio.NewScanner(file)

	// Map to store word frequencies
	wordFreq := make(map[string]int)

	// Iterate over each line in the file
	for scanner.Scan() {
		// Split each line into words
		words := strings.Fields(scanner.Text())

		// Count frequency of each word
		for _, word := range words {
			wordFreq[word]++
		}
	}

	// Convert the map to a slice of word-frequency pairs
	var wordFreqSlice []WordFreq
	for word, freq := range wordFreq {
		wordFreqSlice = append(wordFreqSlice, WordFreq{word, freq})
	}

	// Sort the word-frequency slice by frequency in ascending order
	sort.Slice(wordFreqSlice, func(i, j int) bool {
		return wordFreqSlice[i].Freq < wordFreqSlice[j].Freq
	})

	// Print the top 10 most frequent words
	numWords := len(wordFreqSlice)
	if numWords > 10 {
		numWords = 10
	}
	for i := numWords - 1; i >= 0; i-- {
		fmt.Printf("%6d %s\n", wordFreqSlice[i].Freq, wordFreqSlice[i].Word)
	}
}

// WordFreq struct to store word-frequency pairs
type WordFreq struct {
	Word string
	Freq int
}

